def square(num1):
  return num1*num1

def add(num1, num2):
  return num1+num2

def subtract(num1, num2):
  return num1-num2

def multiply(num1, num2):
  return num1*num2

defdivide(num1, num2):
return num1/num2